const elementValidator = {
  exists: (element, id) => {
    if (!element) {
      logMessage('ОШИБКА', `Не удалось найти элемент с id="${id}".`)
      return false
    }
    return true
  },
  
  isEmpty: element => {
    if (element.isContentEditable) {
      return element.innerText.trim() === ''
    }
    
    if (element.value !== undefined) {
      return element.value.trim() === ''
    }
    
    return element.textContent.trim() === ''
  }
}

const cleaningStrategies = {
  clearInput: element => { element.value = '' },
  clearContent: element => { element.textContent = '' },
  clearEditable: element => { element.innerHTML = '' }
}

const getCleaningStrategy = element => {
  if (element.isContentEditable) return cleaningStrategies.clearEditable
  return element.value !== undefined 
    ? cleaningStrategies.clearInput 
    : cleaningStrategies.clearContent
}

function clearElement(targetId) {
  const element = document.getElementById(targetId)
  
  if (!elementValidator.exists(element, targetId)) return
  if (elementValidator.isEmpty(element)) {
    logMessage('ИНФО', 'Нечего очищать.')
    return
  }

  getCleaningStrategy(element)(element)
  logMessage('ИНФО', 'Успешно очищен.')
}

const handlerFactory = (buttonId, targetId) => ({
  init: () => {
    const button = document.getElementById(buttonId)
    button?.addEventListener('click', () => clearElement(targetId))
  }
})

document.addEventListener('DOMContentLoaded', () => {
  handlerFactory('clear-commands', 'generated-commands').init()
  handlerFactory('clear-links', 'links').init()
  
  handlerFactory('clean-main-server-1', 'my-server-1-links').init()
  handlerFactory('clean-main-server-2', 'my-server-2-links').init()
  handlerFactory('clean-main-server-3', 'my-server-3-links').init()
  handlerFactory('clean-main-server-4', 'my-server-4-links').init()
  handlerFactory('clean-main-server-5', 'my-server-5-links').init()
  handlerFactory('clean-main-server-6', 'my-server-6-links').init()
  handlerFactory('clean-main-server-7', 'my-server-7-links').init()
  handlerFactory('clean-main-server-8', 'my-server-8-links').init()  
})