function copyCommands() {
    const resultElement = document.getElementById('result');
    if (!resultElement) {
        logMessage('ОШИБКА', 'Элемент с id="result" не найден.');
        return;
    }

    const text = resultElement.innerText;
    const lines = text.split('\n');

    const commands = lines
        .filter(line => line.trim().startsWith('Команда:'))
        .map(line => line.split('Команда:')[1].trim())
        .filter(cmd => cmd !== '');

    if (commands.length === 0) {
        logMessage('ИНФО', 'Нет видимых команд для копирования.');
        return;
    }

    const commandsText = commands.join('\n');

    if (navigator.clipboard) {
        navigator.clipboard.writeText(commandsText)
            .then(() => logMessage('ИНФО', 'Команды скопированы!'))
            .catch(err => {
                logMessage('ОШИБКА', `Ошибка копирования: ${err}`);
                fallbackCopyText(commandsText);
            });
    } else {
        fallbackCopyText(commandsText);
    }
}

function fallbackCopyText(text) {
    const textarea = document.createElement('textarea');
    textarea.value = text;
    textarea.style.position = 'fixed';
    textarea.style.opacity = '0';
    document.body.appendChild(textarea);
    textarea.select();

    try {
        document.execCommand('copy');
        logMessage('ИНФО', 'Команды скопированы!');
    } catch (err) {
        logMessage('ОШИБКА', `Копирование не удалось: ${err}`);
    }

    document.body.removeChild(textarea);
}

document.getElementById('copy-result-commands')?.addEventListener('click', copyCommands);