function copyElementContent(elementId) {
  const element = document.getElementById(elementId);
  
  if (!element) {
    logMessage('ОШИБКА', `Элемент с id="${elementId}" не найден.`);
    return;
  }

  let text;
  if (element instanceof HTMLInputElement || element instanceof HTMLTextAreaElement) {
    text = element.value.trim();
  } else {
    text = element.innerText.trim();
  }
  
  if (!text) {
    logMessage('ИНФО', 'Нечего копировать.');
    return;
  }

  if (navigator.clipboard) {
    navigator.clipboard.writeText(text)
      .then(() => logMessage('ИНФО', 'Содержимое успешно скопировано.'))
      .catch(err => {
        fallbackCopyText(text);
      });
  } else {
    fallbackCopyText(text);
  }
}

function fallbackCopyText(text) {
  const textarea = document.createElement('textarea');
  textarea.value = text;
  textarea.style.position = 'fixed';
  textarea.style.opacity = '0';
  textarea.style.left = '-9999px';
  
  document.body.appendChild(textarea);
  textarea.select();
  textarea.focus();
  
  try {
    const successful = document.execCommand('copy');
    if (successful) {
      logMessage('ИНФО', 'Содержимое успешно скопировано.');
    } else {
      logMessage('ОШИБКА', 'Не удалось скопировать.');
    }
  } catch (err) {
    logMessage('ОШИБКА', `Ошибка execCommand: ${err}`);
  }
  
  document.body.removeChild(textarea);
}

function initCopyButtons() {
  const buttons = {
    'copy-pac-link': 'pac-link',
    'copy-result': 'result',
    'copy-links': 'links',
    'copy-generated-commands': 'generated-commands',
		
    'copy-main-server-1': 'my-server-1-links',
    'copy-main-server-2': 'my-server-2-links',
    'copy-main-server-3': 'my-server-3-links',
    'copy-main-server-4': 'my-server-4-links',
    'copy-main-server-5': 'my-server-5-links',
    'copy-main-server-6': 'my-server-6-links',
    'copy-main-server-7': 'my-server-7-links',
    'copy-main-server-8': 'my-server-8-links'	
  };

  Object.entries(buttons).forEach(([buttonId, elementId]) => {
    const button = document.getElementById(buttonId);
    if (button) {
      button.addEventListener('click', () => copyElementContent(elementId));
    } else {
      logMessage('ОШИБКА', `Кнопка с id="${buttonId}" не найдена.`);
    }
  });
}

document.addEventListener('DOMContentLoaded', initCopyButtons);