document.addEventListener('DOMContentLoaded', () => {
    for (let server = 1; server <= 8; server++) {
        const button = document.getElementById(`save-domains-main-server-${server}`);
        if (button) {
            button.addEventListener('click', () => saveDomains(server));
        }
    }
});

async function saveDomains(server) {
    const textElement = document.getElementById(`my-server-${server}-links`);
    if (!textElement) {
        logMessage('ОШИБКА', `Элемент для сервера ${server} не найден.`);
        return;
    }

    const text = textElement.innerText
        .split('\n')
        .map(line => line.trim())
        .filter(line => line !== '')
        .join('\n');

    const data = { server, text };

    try {
        const response = await fetch('save_domains.php', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
                'Accept-Charset': 'UTF-8',
                'Cache-Control': 'no-cache, no-store, must-revalidate',
                'Pragma': 'no-cache'
            },
            body: JSON.stringify(data)
        });

        if (!response.ok) {
            logMessage('ОШИБКА', `Ошибка HTTP: ${response.status}`);
            return;
        }

        const result = await response.json();
        logMessage(result.status ? 'ИНФО' : 'ОШИБКА', result.message);

        if (result.status && result.cleaned) {
            const cleanedText = result.cleaned.join('\n');
            textElement.textContent = cleanedText;
        }
    } catch (error) {
        logMessage('ОШИБКА', `Ошибка при сохранении: ${error.message}`);
    }
}