<?php
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: POST, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type");
header("Cache-Control: no-store, no-cache, must-revalidate");
header("Pragma: no-cache");
header('Content-Type: application/json; charset=utf-8');

if ($_SERVER['REQUEST_METHOD'] === 'OPTIONS') {
    http_response_code(204);
    header('Content-Length: 0');
    exit();
}

function generate_json_answer($response) {
    echo json_encode($response, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE);
    exit();
}

function is_port_in_use($port) {
    $cmd = "lsof -i :$port -sTCP:LISTEN -t";
    $output = trim(shell_exec($cmd));
    return !empty($output);
}

function get_process_pid($port) {
    $cmd = "lsof -i :$port -sTCP:LISTEN -t";
    $output = trim(shell_exec($cmd));
    return $output ? $output : null;
}

function get_process_info($pid) {
    if (!$pid) {
        return null;
    }
    $exe_path = @readlink("/proc/$pid/exe");
    if (!$exe_path) {
        return null;
    }
    $cmdline = @file_get_contents("/proc/$pid/cmdline");
    if (!$cmdline) {
        return null;
    }
    $cmdline = str_replace("\0", " ", $cmdline);
    return [
        'exe' => $exe_path,
        'cmdline' => trim($cmdline)
    ];
}

function process_arguments($cmdline, $port) {
    $port_flag = "--port $port";
    $pos = strpos($cmdline, $port_flag);
    $arguments = '';
    if ($pos !== false) {
        $start = $pos + strlen($port_flag);
        while ($start < strlen($cmdline) && ctype_space($cmdline[$start])) {
            $start++;
        }
        $arguments = $start < strlen($cmdline) ? substr($cmdline, $start) : '';
    }

    $hosts_file = false;
    if (!empty($arguments)) {
        $args_array = preg_split('/\s+/', trim($arguments));
        $i = 0;
        while ($i < count($args_array)) {
            if ($args_array[$i] === '--hosts') {
                $hosts_file = true;
                if ($i + 1 < count($args_array)) {
                    array_splice($args_array, $i, 2);
                } else {
                    array_splice($args_array, $i, 1);
                }
            } else {
                $i++;
            }
        }
        $arguments = implode(' ', $args_array);
    }
    return [
        'arguments' => $arguments,
        'hosts_file' => $hosts_file
    ];
}

function start_process($real_path, $port, $args) {
    $command = "nohup \"$real_path\" --port $port $args > /dev/null 2>&1 &";
    shell_exec($command);
}

function kill_process($pid) {
    $command = "kill -9 $pid";
    shell_exec($command);
}

function get_current_state($real_path, $port) {
    if (!is_port_in_use($port)) {
        return [
            'state' => 'free',
            'pid' => null,
            'hosts_file' => false,
            'arguments' => ''
        ];
    }
    $pid = get_process_pid($port);
    if (!$pid) {
        return [
            'state' => 'in_use_by_others',
            'pid' => null,
            'hosts_file' => false,
            'arguments' => ''
        ];
    }
    $process_info = get_process_info($pid);
    if ($process_info && $process_info['exe'] === $real_path && strpos($process_info['cmdline'], "--port $port") !== false) {
        $args_status = process_arguments($process_info['cmdline'], $port);
        return [
            'state' => 'in_use_by_us',
            'pid' => $pid,
            'hosts_file' => $args_status['hosts_file'],
            'arguments' => $args_status['arguments']
        ];
    }
    return [
        'state' => 'in_use_by_others',
        'pid' => null,
        'hosts_file' => false,
        'arguments' => ''
    ];
}

$input_data = file_get_contents('php://input');
$request_data = json_decode($input_data, true);

if (json_last_error() !== JSON_ERROR_NONE) {
    generate_json_answer([
        'action' => null,
        'real_file_path' => null,
        'port' => null,
        'hosts_file' => null,
        'arguments' => null,
        'pid' => null,
        'message' => 'Некорректный формат JSON.',
        'result' => false
    ]);
}

$required_fields = ['action', 'real_file_path', 'port'];
foreach ($required_fields as $field) {
    if (!isset($request_data[$field])) {
        generate_json_answer([
            'action' => $request_data['action'] ?? null,
            'real_file_path' => $request_data['real_file_path'] ?? null,
            'port' => $request_data['port'] ?? null,
            'hosts_file' => null,
            'arguments' => null,
            'pid' => null,
            'message' => "Отсутствует обязательное поле: $field.",
            'result' => false
        ]);
    }
}

$action = $request_data['action'];
$real_file_path = $request_data['real_file_path'];
$port = $request_data['port'];
$arguments = $request_data['arguments'] ?? '';

$allowed_actions = ['check', 'check_and_start', 'check_and_kill'];
if (!in_array($action, $allowed_actions)) {
    generate_json_answer([
        'action' => $action,
        'real_file_path' => $real_file_path,
        'port' => $port,
        'hosts_file' => null,
        'arguments' => null,
        'pid' => null,
        'message' => 'Недопустимое действие.',
        'result' => false
    ]);
}

if (empty(trim($real_file_path)) || !file_exists($real_file_path) || !is_file($real_file_path)) {
    $message = empty(trim($real_file_path)) ? 'Путь к программе пустой или состоит из пробелов.' :
               (!file_exists($real_file_path) ? 'Файл программы не найден.' : 'Указанный путь не является файлом.');
    generate_json_answer([
        'action' => $action,
        'real_file_path' => $real_file_path,
        'port' => $port,
        'hosts_file' => null,
        'arguments' => null,
        'pid' => null,
        'message' => $message,
        'result' => false
    ]);
}

if (!is_numeric($port) || intval($port) != $port || $port < 1 || $port > 65535) {
    $message = (!is_numeric($port) || intval($port) != $port) ? 'Порт должен быть целым числом.' :
               'Порт находится не в пределах 1-65535.';
    generate_json_answer([
        'action' => $action,
        'real_file_path' => $real_file_path,
        'port' => $port,
        'hosts_file' => null,
        'arguments' => null,
        'pid' => null,
        'message' => $message,
        'result' => false
    ]);
}

$port = intval($port);

$state_info = get_current_state($real_file_path, $port);

if ($action === 'check') {
    $message = $state_info['state'] === 'free' ? 'Порт свободен.' :
               ($state_info['state'] === 'in_use_by_us' ? 'Порт занят нашей программой.' : 'Порт занят другой программой.');
    generate_json_answer([
        'action' => $action,
        'real_file_path' => $real_file_path,
        'port' => $port,
        'hosts_file' => $state_info['hosts_file'],
        'arguments' => $state_info['arguments'],
        'pid' => $state_info['pid'],
        'message' => $message,
        'result' => true
    ]);
} elseif ($action === 'check_and_start') {
    if ($state_info['state'] === 'in_use_by_us') {
        generate_json_answer([
            'action' => $action,
            'real_file_path' => $real_file_path,
            'port' => $port,
            'hosts_file' => $state_info['hosts_file'],
            'arguments' => $state_info['arguments'],
            'pid' => $state_info['pid'],
            'message' => 'Процесс уже запущен.',
            'result' => true
        ]);
    } elseif ($state_info['state'] === 'in_use_by_others') {
        generate_json_answer([
            'action' => $action,
            'real_file_path' => $real_file_path,
            'port' => $port,
            'hosts_file' => false,
            'arguments' => '',
            'pid' => null,
            'message' => 'Порт занят другой программой.',
            'result' => false
        ]);
    } else {
        start_process($real_file_path, $port, $arguments);
        $max_wait = 5;
        for ($i = 0; $i < $max_wait; $i++) {
            sleep(1);
            $state_info = get_current_state($real_file_path, $port);
            if ($state_info['state'] === 'in_use_by_us') {
                break;
            }
        }
        $result = $state_info['state'] === 'in_use_by_us';
        $message = $result ? 'Успешно запущен процесс.' : 'Не удалось запустить процесс после 3 секунд ожидания.';
        generate_json_answer([
            'action' => $action,
            'real_file_path' => $real_file_path,
            'port' => $port,
            'hosts_file' => $state_info['hosts_file'],
            'arguments' => $state_info['arguments'],
            'pid' => $state_info['pid'],
            'message' => $message,
            'result' => $result
        ]);
    }
} elseif ($action === 'check_and_kill') {
    if ($state_info['state'] === 'free') {
        generate_json_answer([
            'action' => $action,
            'real_file_path' => $real_file_path,
            'port' => $port,
            'hosts_file' => false,
            'arguments' => '',
            'pid' => null,
            'message' => 'Порт уже свободен.',
            'result' => true
        ]);
    } elseif ($state_info['state'] === 'in_use_by_others') {
        generate_json_answer([
            'action' => $action,
            'real_file_path' => $real_file_path,
            'port' => $port,
            'hosts_file' => false,
            'arguments' => '',
            'pid' => null,
            'message' => 'Порт занят другой программой, не могу остановить.',
            'result' => false
        ]);
    } else {
        $pid = $state_info['pid'];
        $max_attempts = 10;
        $attempts = 0;
        $port_free = false;
        while (!$port_free && $attempts < $max_attempts) {
            $attempts++;
            kill_process($pid);
            sleep(1);
            $state_info = get_current_state($real_file_path, $port);
            $port_free = $state_info['state'] === 'free';
        }
        $result = $port_free;
        $message = $result ? "Процесс успешно остановлен после $attempts попыток." :
                   "Не удалось остановить процесс после $max_attempts попыток.";
        generate_json_answer([
            'action' => $action,
            'real_file_path' => $real_file_path,
            'port' => $port,
            'hosts_file' => $state_info['hosts_file'],
            'arguments' => $state_info['arguments'],
            'pid' => $state_info['pid'],
            'message' => $message,
            'result' => $result
        ]);
    }
}

?>