<?php
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: POST, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type");
header("Cache-Control: no-store, no-cache, must-revalidate");
header("Pragma: no-cache");
header('Content-Type: application/json; charset=utf-8');

if ($_SERVER['REQUEST_METHOD'] === 'OPTIONS') {
    http_response_code(204);
    header('Content-Length: 0');
    exit();
}

function generate_json_answer($response) {
    echo json_encode($response, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE);
    exit();
}

function is_port_in_use($port) {
    $cmd = "netstat -ano | findstr /C:\":$port \" | findstr LISTENING";
    $output = shell_exec($cmd);
    return !empty(trim($output));
}

function get_process_output($real_path, $port) {
    $escaped_path = str_replace("'", "''", $real_path);
    $command = 'powershell -NoProfile -ExecutionPolicy Bypass -Command ';
    $command .= '"Get-CimInstance Win32_Process | ';
    $command .= 'Where-Object { ';
    $command .= '$_.ExecutablePath -eq \'' . $escaped_path . '\' -and ';
    $command .= '$_.CommandLine -like \'*--port ' . $port . '*\' } | ';
    $command .= 'ForEach-Object { ';
    $command .= '\'<PID>\' + $_.ProcessId + \'</PID>\'; ';
    $command .= '\'<ARGS>\' + $_.CommandLine + \'</ARGS>\' ';
    $command .= '}" 2>&1';
    return shell_exec($command);
}

function parse_process_output($output) {
    if (empty(trim($output))) {
        return null;
    }
    if (preg_match('/<PID>(\d+)<\/PID>/', $output, $pid_matches) &&
        preg_match('/<ARGS>(.+)<\/ARGS>/', $output, $args_matches)) {
        return [
            'pid' => $pid_matches[1],
            'command_line' => $args_matches[1]
        ];
    }
    return null;
}

function process_arguments($command_line, $port) {
    $port_flag = "--port $port";
    $pos = strpos($command_line, $port_flag);
    $arguments = '';
    if ($pos !== false) {
        $start = $pos + strlen($port_flag);
        while ($start < strlen($command_line) && ctype_space($command_line[$start])) {
            $start++;
        }
        $arguments = $start < strlen($command_line) ? substr($command_line, $start) : '';
    }

    $hosts_file = false;
    if (!empty($arguments)) {
        $args_array = preg_split('/\s+/', trim($arguments));
        $i = 0;
        while ($i < count($args_array)) {
            if ($args_array[$i] === '--hosts') {
                $hosts_file = true;
                if ($i + 1 < count($args_array)) {
                    array_splice($args_array, $i, 2);
                } else {
                    array_splice($args_array, $i, 1);
                }
            } else {
                $i++;
            }
        }
        $arguments = implode(' ', $args_array);
    }
    return [
        'arguments' => $arguments,
        'hosts_file' => $hosts_file
    ];
}

function get_process_info($real_path, $port) {
    $output = get_process_output($real_path, $port);
    if (empty(trim($output))) {
        return null;
    }
    $process_info = parse_process_output($output);
    if ($process_info === null) {
        return null;
    }
    $args_status = process_arguments($process_info['command_line'], $port);
    return [
        'pid' => $process_info['pid'],
        'arguments' => $args_status['arguments'],
        'hosts_file' => $args_status['hosts_file']
    ];
}

function start_process($real_path, $port, $args) {
    $command = "start /B \"\" \"$real_path\" --port $port $args > nul 2>&1";
    pclose(popen($command, 'r'));
}

function kill_process($pid) {
    $command = "taskkill /F /PID $pid 2>&1";
    $output = shell_exec($command);
    return strpos($output, 'SUCCESS') !== false;
}

function get_current_state($real_path, $port) {
    if (!is_port_in_use($port)) {
        return [
            'state' => 'free',
            'pid' => null,
            'hosts_file' => false,
            'arguments' => ''
        ];
    }
    $process_info = get_process_info($real_path, $port);
    if ($process_info !== null) {
        return [
            'state' => 'in_use_by_us',
            'pid' => $process_info['pid'],
            'hosts_file' => $process_info['hosts_file'],
            'arguments' => $process_info['arguments']
        ];
    }
    return [
        'state' => 'in_use_by_others',
        'pid' => null,
        'hosts_file' => false,
        'arguments' => ''
    ];
}

$input_data = file_get_contents('php://input');
$request_data = json_decode($input_data, true);

if (json_last_error() !== JSON_ERROR_NONE) {
    generate_json_answer([
        'action' => null,
        'real_file_path' => null,
        'port' => null,
        'hosts_file' => null,
        'arguments' => null,
        'pid' => null,
        'message' => 'Некорректный формат JSON.',
        'result' => false
    ]);
}

$required_fields = ['action', 'real_file_path', 'port'];
foreach ($required_fields as $field) {
    if (!isset($request_data[$field])) {
        generate_json_answer([
            'action' => $request_data['action'] ?? null,
            'real_file_path' => $request_data['real_file_path'] ?? null,
            'port' => $request_data['port'] ?? null,
            'hosts_file' => null,
            'arguments' => null,
            'pid' => null,
            'message' => "Отсутствует обязательное поле: $field.",
            'result' => false
        ]);
    }
}

$action = $request_data['action'];
$real_file_path = $request_data['real_file_path'];
$port = $request_data['port'];
$arguments = $request_data['arguments'] ?? '';

$allowed_actions = ['check', 'check_and_start', 'check_and_kill'];
if (!in_array($action, $allowed_actions)) {
    generate_json_answer([
        'action' => $action,
        'real_file_path' => $real_file_path,
        'port' => $port,
        'hosts_file' => null,
        'arguments' => null,
        'pid' => null,
        'message' => 'Недопустимое действие.',
        'result' => false
    ]);
}

if (empty(trim($real_file_path)) || !file_exists($real_file_path) || !is_file($real_file_path)) {
    $message = empty(trim($real_file_path)) ? 'Путь к программе пустой или состоит из пробелов.' :
               (!file_exists($real_file_path) ? 'Файл программы не найден.' : 'Указанный путь не является файлом.');
    generate_json_answer([
        'action' => $action,
        'real_file_path' => $real_file_path,
        'port' => $port,
        'hosts_file' => null,
        'arguments' => null,
        'pid' => null,
        'message' => $message,
        'result' => false
    ]);
}

if (!is_numeric($port) || intval($port) != $port || $port < 1 || $port > 65535) {
    $message = (!is_numeric($port) || intval($port) != $port) ? 'Порт должен быть целым числом.' :
               'Порт находится не в пределах 1-65535.';
    generate_json_answer([
        'action' => $action,
        'real_file_path' => $real_file_path,
        'port' => $port,
        'hosts_file' => null,
        'arguments' => null,
        'pid' => null,
        'message' => $message,
        'result' => false
    ]);
}

$port = intval($port);

$state_info = get_current_state($real_file_path, $port);

if ($action === 'check') {
    $message = $state_info['state'] === 'free' ? 'Порт свободен.' :
               ($state_info['state'] === 'in_use_by_us' ? 'Порт занят нашей программой.' : 'Порт занят другой программой.');
    generate_json_answer([
        'action' => $action,
        'real_file_path' => $real_file_path,
        'port' => $port,
        'hosts_file' => $state_info['hosts_file'],
        'arguments' => $state_info['arguments'],
        'pid' => $state_info['pid'],
        'message' => $message,
        'result' => true
    ]);
} elseif ($action === 'check_and_start') {
    if ($state_info['state'] === 'in_use_by_us') {
        generate_json_answer([
            'action' => $action,
            'real_file_path' => $real_file_path,
            'port' => $port,
            'hosts_file' => $state_info['hosts_file'],
            'arguments' => $state_info['arguments'],
            'pid' => $state_info['pid'],
            'message' => 'Процесс уже запущен.',
            'result' => true
        ]);
    } elseif ($state_info['state'] === 'in_use_by_others') {
        generate_json_answer([
            'action' => $action,
            'real_file_path' => $real_file_path,
            'port' => $port,
            'hosts_file' => false,
            'arguments' => '',
            'pid' => null,
            'message' => 'Порт занят другой программой.',
            'result' => false
        ]);
    } else {
        start_process($real_file_path, $port, $arguments);
        $max_wait = 5;
        for ($i = 0; $i < $max_wait; $i++) {
            sleep(1);
            $state_info = get_current_state($real_file_path, $port);
            if ($state_info['state'] === 'in_use_by_us') {
                break;
            }
        }
        $result = $state_info['state'] === 'in_use_by_us';
        $message = $result ? 'Успешно запущен процесс.' : 'Не удалось запустить процесс после 3 секунд ожидания.';
        generate_json_answer([
            'action' => $action,
            'real_file_path' => $real_file_path,
            'port' => $port,
            'hosts_file' => $state_info['hosts_file'],
            'arguments' => $state_info['arguments'],
            'pid' => $state_info['pid'],
            'message' => $message,
            'result' => $result
        ]);
    }
} elseif ($action === 'check_and_kill') {
    if ($state_info['state'] === 'free') {
        generate_json_answer([
            'action' => $action,
            'real_file_path' => $real_file_path,
            'port' => $port,
            'hosts_file' => false,
            'arguments' => '',
            'pid' => null,
            'message' => 'Порт уже свободен.',
            'result' => true
        ]);
    } elseif ($state_info['state'] === 'in_use_by_others') {
        generate_json_answer([
            'action' => $action,
            'real_file_path' => $real_file_path,
            'port' => $port,
            'hosts_file' => false,
            'arguments' => '',
            'pid' => null,
            'message' => 'Порт занят другой программой, не могу остановить.',
            'result' => false
        ]);
    } else {
        $pid = $state_info['pid'];
        $max_attempts = 10;
        $attempts = 0;
        $port_free = false;
        while (!$port_free && $attempts < $max_attempts) {
            $attempts++;
            kill_process($pid);
            sleep(1);
            $state_info = get_current_state($real_file_path, $port);
            $port_free = $state_info['state'] === 'free';
        }
        $result = $port_free;
        $message = $result ? "Процесс успешно остановлен после $attempts попыток." :
                   "Не удалось остановить процесс после $max_attempts попыток.";
        generate_json_answer([
            'action' => $action,
            'real_file_path' => $real_file_path,
            'port' => $port,
            'hosts_file' => $state_info['hosts_file'],
            'arguments' => $state_info['arguments'],
            'pid' => $state_info['pid'],
            'message' => $message,
            'result' => $result
        ]);
    }
}

?>